########################################
### OSU / Dept of Political Science  ###
### PRISM Workshop: MLE Extensions   ###
### March 28, 2014                   ###
### Interactions in Nonlinear Models ###
### (Replication code for plots in   ###
### NAFTA example )                  ###  
########################################


########################
### Model estimation ###
########################

#Load data
library(foreign)
nafta <- read.dta("nafta.dta")

#Estimate logit model with multiplicative interaction term
model1 <- glm(vote ~ democrat + cope93 + democrat*cope93 + pcthispc, data=nafta, family=binomial(logit))
summary(model1)


##################################################################
### Bootstrapping predicted probabilities and CIs for plotting ###
##################################################################

##########################################################################################
### Plot (1):                                                                           ##
### Predicted probability of voting for NAFTA as a function of pro-labor score (COPE93) ##
### for Democrats (continuous line) and Republicans (dashed line)                       ##
##########################################################################################


#Simulated values of cope93 (pro-labor measure) to be used to compute predicted probabilities
summary(nafta)
cope93.range <- seq(min(nafta$cope93), max(nafta$cope93), by=1) #using full range of cope93

#Setting parameters for bootstrap

n <- length(nafta$vote) #number of observations in the sample
B <- 1000 #number of bootstraps resamples

out1 <- matrix(0, nrow=length(cope93.range), ncol=3) #final output matrix for Dem=1
out2 <- matrix(0, nrow=length(cope93.range), ncol=3) #final output matrix for Dem=0

#Bootstrapping predictions for Democrats (democrat=1)

for(i in 1:length(cope93.range)){
  cope93.sim <- cope93.range[i] #for computing model predictions for each value of cope93 (pro-labor measure)
  avg.bs <- matrix(NA,B,1) #a storage matrix for the bootstrap estimates
  for(b in 1:B){
    resample <- sample(nafta$case,n,replace=T) #resample from data with replacement
    glm.bs <- glm(vote ~ democrat + cope93 + democrat*cope93 + pcthispc, data=nafta[is.element(nafta$case, resample),],family=binomial(logit)) #estimate logit model
    avg.bs[b,] <- plogis(glm.bs$coef[1]*1 + glm.bs$coef[2]*1 + glm.bs$coef[3]*cope93.range[i] + glm.bs$coef[4]*mean(nafta$pcthispc) + glm.bs$coef[5]*cope93.range[i]*1) #compute predicted probabilities from model
    rm(resample)
  }
  out1[i,] <- c(mean(avg.bs, na.rm=TRUE), quantile(avg.bs, c(0.025, 0.975), na.rm=TRUE)) #use mean for predicted probability; 2.5 and 97.5 percentiles for 95% CIS
  rm(avg.bs)
}

#Bootstrapping predictions for Republicans (democrat=0)

for(i in 1:length(cope93.range)){
  cope93.sim <- cope93.range[i]
  avg.bs <- matrix(NA,B,1)
  for(b in 1:B){
    resample <- sample(nafta$case,n,replace=T)
    glm.bs <- glm(vote ~ democrat + cope93 + democrat*cope93 + pcthispc, data=nafta[is.element(nafta$case, resample),],family=binomial(logit))
    avg.bs[b,] <- plogis(glm.bs$coef[1]*1 + glm.bs$coef[2]*0 + glm.bs$coef[3]*cope93.range[i] + glm.bs$coef[4]*mean(nafta$pcthispc) + glm.bs$coef[5]*cope93.range[i]*0)
    rm(resample)
  }
  out2[i,] <- c(mean(avg.bs, na.rm=TRUE), quantile(avg.bs, c(0.025, 0.975), na.rm=TRUE))
  rm(avg.bs)
}

#Plot
library(ggplot2)

out1 <- data.frame(out1) #ggplot2 requires data frames
out2 <- data.frame(out2)

ggplot(out1, aes(x= cope93.range, y=out1[,1])) +
  geom_line(aes(y=out1[,1]), linetype=1, size=1.2) +
  geom_ribbon(aes(ymin=out1[,2], ymax=out1[,3]), alpha=.3) +
  geom_line(aes(y=out2[,1]), linetype=2, size=1.2) +
  geom_ribbon(aes(ymin=out2[,2], ymax=out2[,3]), alpha=.3) +
  xlab("Pro-labor score") +
  ylab("Predicted probability of voting for NATA") +
  theme(legend.position="bottom")



#########################################################################################
### Plot (2):                                                                          ##
### Discrete change in predicted probability of voting for NAFTA from a mean-centered, ##
### 2-standard-deviation change in COPE93 for Democrats and Republicans.               ##
### Vertical bars are 95% bootstrapped CIs.                                            ##
#########################################################################################

#Computing predicted probabilities and bootstrapped CIs for plotting standard-deviation changes in COPE93

n <- length(nafta$vote) #number of observations in the sample
B <- 1500 #number of bootstraps resamples

#Storage matrices
avg.bs <- matrix(NA,B,1)
dem.sim <- c(1,0) #values of 'democrat' variable to be used
out3 <- matrix(NA,length(dem.sim),3) #output matrix

for (i in 1:length(dem.sim)){
  
for(b in 1:B){
  resample <- sample(nafta$case,n,replace=T)
  glm.bs <- glm(vote ~ democrat + cope93 + democrat*cope93 + pcthispc, data=nafta[is.element(nafta$case, resample),],family=binomial(logit))
  avg.bs[b,] <- plogis(glm.bs$coef[1]*1 + glm.bs$coef[2]*dem.sim[i] + glm.bs$coef[3]*(mean(nafta$cope93)+(sd(nafta$cope93))) + glm.bs$coef[4]*mean(nafta$pcthispc) + glm.bs$coef[5]*(mean(nafta$cope93)+(sd(nafta$cope93)))*dem.sim[i]) - plogis(glm.bs$coef[1]*1 + glm.bs$coef[2]*dem.sim[i] + glm.bs$coef[3]*(mean(nafta$cope93)-(sd(nafta$cope93))) + glm.bs$coef[4]*mean(nafta$pcthispc) + glm.bs$coef[5]*(mean(nafta$cope93)-(sd(nafta$cope93)))*dem.sim[i])
  drop(resample)
}
avg.bs <- na.omit(avg.bs)
out3[i,1] <- mean(avg.bs, na.rm=TRUE)
out3[i,2:3] <- quantile(avg.bs, c(0.025, 0.975), na.rm=TRUE)
}

#plot
out3 <- data.frame(out3) #ggplot2 requires a data frame object
out3$dem.sim <- as.factor(dem.sim)

p <- ggplot(out3, aes(x=out3[,4], y=out3[,1]))
p + geom_point(size=4) + geom_pointrange(aes(ymin=out3[,2], ymax=out3[,3]), size=1) +
  xlab("Democrat") +
  ylab("Predicted change in probability of voting for NATA") +
  scale_x_discrete(breaks = 0:1, labels=c("Republican","Democrat")) +
  xlab(NULL)

